<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.codevision.io
 * @since      1.0.0
 *
 * @package    Codevision_Elementor_Cmb
 * @subpackage Codevision_Elementor_Cmb/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Codevision_Elementor_Cmb
 * @subpackage Codevision_Elementor_Cmb/admin
 * @author     Codevision <info@codevision.io>
 */
class Codevision_Elementor_Cmb_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version The version of this plugin.
     *
     * @since    1.0.0
     */
    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version     = $version;

    }

    /**
     * Add Menu to Page
     */
    public function add_admin_page() {

        do_action('codevision/cmb/add_admin_page/before');

        $img_path = 'data:image/png;base64,' . base64_encode( file_get_contents( CMB_ROOT . '/admin/img/codevision-elementor-breakpoints.png' ) );

        acf_add_options_page( array(
            'page_title' => apply_filters('codevision/cmb/page_title', 'Breakpoints'),
            'menu_title' => apply_filters('codevision/cmb/menu_title', 'Breakpoints'),
            'menu_slug'  => CMB_SLUG,
            'capability' => 'manage_options',
            'icon_url'   => $img_path,
            'redirect'   => true,
        ) );

        acf_add_options_sub_page( array(
            'page_title'  => __( 'Settings', 'codevision-elementor-cmb' ),
            'menu_title'  => __( 'Settings', 'codevision-elementor-cmb' ),
            'parent_slug' => CMB_SLUG,
            'slug'        => 'acf-options-cmb-settings',
            'icon_url'    => $img_path,
        ) );

        do_action( 'codevision/cmb/add_admin_page', $img_path );
    }

    public function enqueue_styles() {

        wp_enqueue_style( $this->plugin_name . '-admin', plugin_dir_url( __FILE__ ) . 'css/codevision-elementor-cmb-admin.css', array(), $this->version, 'all' );

        $this->enqueue_css( 'css/bootstrap.min.css', 'acf-options-cmb-settings' );
    }

    public function enqueue_scripts() {

        $this->enqueue_script( 'js/codevision-elementor-cmb-acf.js',
            'acf-options-cmb-settings',
            ['jquery'],
            true,
            [
                'cmb_locale' => [
                    'wordpress_locale' => get_locale(),
                ],
            ] );

        $this->enqueue_script( 'js/bootstrap.min.js', 'acf-options-cmb-settings' );

        do_action('codevision/cmb/enqueue_scripts');
    }

    public function elementor_enqueue_scripts() {

        $this->enqueue_script( 'js/codevision-elementor-cmb-admin.js' );

        do_action('codevision/cmb/enqueue_scripts');
    }

    public function elementor_enqueue_styles() {

        wp_enqueue_style( $this->plugin_name . '-editor', plugin_dir_url( __FILE__ ) . 'css/codevision-elementor-editor.css', array(), $this->version, 'all' );

        if ( file_exists( __DIR__ . '/css/codevision-elementor-cmb-custom.css' ) ) {
            $sha = hash_file( 'sha256', __DIR__ . '/css/codevision-elementor-cmb-custom.css' );
            wp_enqueue_style( $this->plugin_name . '-custom', plugin_dir_url( __FILE__ ) . 'css/codevision-elementor-cmb-custom.css', array(), $sha, 'all' );
        }

        do_action('codevision/cmb/enqueue_styles');
    }

    public function init() {

        if ( ! did_action( 'elementor/loaded' ) ) {
            add_action( 'admin_notices', [ $this, 'notice_missing_elementor' ] );

            return;
        }

        $installed_version = get_option( CMB_SLUG . '_version', false );

        if ( CMB_DEBUG || $installed_version !== CMB_VERSION ) {
            do_action( 'codevision/cmb/activate' );

            update_option( CMB_SLUG . '_version', CMB_VERSION );
        }

        do_action( 'codevision/cmb/init' );
    }

    public function notice_missing_elementor() {

        $message = sprintf( esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'codevision-elementor-cmb' ),
            '<strong>' . CMB_NAME . '</strong>',
            '<strong>Elementor</strong>' );

        printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
    }

    public function on_acf_save( $post_id ) {

        $screen = get_current_screen();

        if ( strpos( $screen->id, 'acf-options-cmb-settings' ) === false ) {
            return;
        }

        do_action( 'codevision/cmb/acf/save_post', $post_id );
    }

    public function on_acf_save_done( $post_id ) {

        $screen = get_current_screen();

        if ( strpos( $screen->id, 'acf-options-cmb-settings' ) === false ) {
            return;
        }

        do_action( 'codevision/cmb/acf/after_save_post', $post_id );
    }

    private function enqueue_script( $file, $current_screen = false, $requirements = [ 'jquery' ], $in_footer = true, $locale = [] ) {

        if ( $current_screen && strpos( get_current_screen()->id, $current_screen ) === false ) {
            return;
        }

        $sha           = hash_file( 'sha256', __DIR__ . '/' . $file );
        $script_handle = sprintf( '%s-%s', $this->plugin_name, $sha );
        $full_file     = plugin_dir_url( __FILE__ ) . $file;

        wp_enqueue_script( $script_handle, $full_file, $requirements, $sha, $in_footer );

        foreach ( $locale as $object_name => $l18n ) {
            wp_localize_script( $script_handle, $object_name, $l18n );
        }

        do_action('codevision/cmb/enqueue_script/js_data', $script_handle, $full_file, $file);
    }

    private function enqueue_css( $file, $current_screen = false, $requirements = [], $media = 'all' ) {

        if ( $current_screen && strpos( get_current_screen()->id, $current_screen ) === false ) {
            return;
        }

        $sha = hash_file( 'sha256', __DIR__ . '/' . $file );


        wp_enqueue_style( sprintf( '%s-%s', $this->plugin_name, $sha ), plugin_dir_url( __FILE__ ) . $file, $requirements, $sha, $media );
    }




}
