<?php


namespace Codevision\Elementor;


use Codevision\Elementor\Mixin\MarginMixin;
use Codevision\Elementor\Mixin\PaddingMixin;

class CustomBreakpoints {

    /**
     * @var string
     */
    private $settings;

    /**
     * @var bool
     */
    private $skip_import_check;

    /**
     * CustomBreakpoints constructor.
     */
    public function __construct() {

        $this->settings          = CMB_ROOT . '/required/acf/settings/devices.json';
        $this->skip_import_check = false;

        ( new PaddingMixin() )->hooks();
        ( new MarginMixin() )->hooks();

        add_action( 'codevision/cmb/activate', [ $this, 'activate' ] );
        add_action( 'codevision/cmb/deactivate', [ $this, 'deactivate' ] );
        add_action( 'codevision/cmb/acf/save_post', [ $this, 'postSaveAction' ] );
        add_action( 'codevision/cmb/init', [ $this, 'init' ] );
        add_action( 'codevision/cmb/enqueue_script/js_data', [ $this, 'enqueueScripts' ] );
        add_action( 'wp_ajax_cmb-save-settings', [ $this, 'ajaxSaveSettings' ] );
    }

    private function get_device_by_setting( $setting, $devices ) {

        foreach ( $devices as $device ) {

            if ( $setting[ 'device_name' ] === self::getDeviceName( $device ) ) {
                return $device;
            }
        }

        return false;
    }

    function ajaxSaveSettings() {

        check_ajax_referer( 'cmb-save-settings', 'security' );

        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }

        $json_data = $_POST[ 'dataToSave' ] ?? '';

        if ( ! $json_data ) {
            return;
        }

        $settings = json_decode( stripslashes( $json_data ), true );

        if ( ! $settings ) {
            return;
        }

        $postId = 0;

        foreach ( $settings as $setting ) {

            if ( ! is_array( $setting ) ) {
                continue;
            }

            if ( ! array_key_exists( 'device_name', $setting ) ) {
                continue;
            }

            $postId = intval( $setting[ 'post_id' ] );

            $this->saveDeviceData( $setting[ 'device_name' ], $setting );
        }

        if ( $postId > 0 ) {
            $this->updateCssFile( $postId );
        }

        wp_send_json( [ 'done' => true ] );
    }


    function enqueueScripts( $script_handle ) {

        wp_localize_script( $script_handle,
            'cmb_nonces',
            array(
                'mixins'        => wp_create_nonce( 'cmb-load-mixins' ),
                'save_settings' => wp_create_nonce( 'cmb-save-settings' ),
                'load_settings' => wp_create_nonce( 'cmb-load-settings' ),
            ) );

        wp_localize_script( $script_handle, 'cmb_mixins', apply_filters( 'codevision/cmb/mixins', [] ) );

    }


    function activate() {

        update_option( 'cmb-is-activated', '1' );
    }

    function deactivate() {

        delete_option( 'cmb-is-activated' );
        delete_transient( 'cmb-generated-dynamic-css-and-js' );

    }

    function init() {

        $activated = get_option( 'cmb-is-activated' );
        $css       = get_transient( 'cmb-generated-dynamic-css-and-js' );
        $devices   = get_field( 'cmb_breakpoint_configuration', 'options' );

        if ( $activated !== '1' && is_array( $devices ) && $css ) {
            return;
        }

        if ( is_array( $devices ) && sizeof( $devices ) > 0 ) {
            $this->postSaveAction();

            return;
        }

        delete_option( 'cmb-is-activated' );

        $this->import();
    }

    /**
     * @param bool $skip_import_check
     *
     * @return $this
     */
    public function setSkipImportCheck( bool $skip_import_check ) {

        $this->skip_import_check = $skip_import_check;

        return $this;
    }

    /**
     * @param string $settings
     *
     * @return CustomBreakpoints $this
     */
    public function setSettings( string $settings ) {

        $this->settings = $settings;

        return $this;
    }

    public function import() {

        $base_settings = json_decode( file_get_contents( $this->settings ), true );

        if ( ! $base_settings ) {
            return $this;
        }

        if ( ! is_array( $base_settings ) ) {
            return $this;
        }

        if ( ! array_key_exists( 'modified', $base_settings ) ) {
            return $this;
        }

        $file_hash = hash_file( 'sha256', $this->settings );

        $last_import = get_option( 'cmb_last_import_' . $file_hash, 0 );

        $devices = get_field( 'cmb_breakpoint_configuration', 'options' );

        if ( ! is_array( $devices ) ) {
            $devices = [];
        }

        if ( $devices && ! CMB_DEBUG && ! $this->skip_import_check && $last_import > $base_settings[ 'modified' ] ) {
            return $this->postSaveAction();
        }

        $base_devices = apply_filters( 'codevision/cmb/imported-devices',
            array_map( function( $device ) {

                if ( array_key_exists( 'is_enabled', $device ) ) {
                    return $device;
                }

                $device[ 'is_enabled' ] = false;

                return $device;
            },
                $base_settings[ 'devices' ] ) );

        $base_device_identifiers = array_map( function( $device ) {

            return $device[ 'identifier' ];
        },
            $base_devices );

        $user_devices = array_filter( $devices,
            function( $device ) use ( $base_device_identifiers ) {

                return ! in_array( $device[ 'name' ], $base_device_identifiers );
            } );

        $merged = array_merge( array_values( $base_devices ), array_values( $user_devices ) );

        $merged = $this->array_unique( $merged );

        $merged = apply_filters( 'codevision/cmb/imported-and-merged-devices', $merged );

        update_field( 'cmb_breakpoint_configuration', $merged, 'options' );

        update_option( 'cmb_last_import_' . $file_hash, time() );

        return $this->postSaveAction();
    }

    private function array_unique( array $array ) {

        $result = [];

        foreach ( $array as $device ) {
            $result[ sha1( $device[ 'name' ] ) ] = $device;
        }

        return array_values( $result );
    }

    private function get_enabled_devices() {

        $devices = get_field( 'cmb_breakpoint_configuration', 'options' );

        if ( ! is_array( $devices ) ) {
            return [];
        }

        return array_filter( $devices,
            function( $device ) {

                return ( $device[ 'is_enabled' ] ?? false ) === true;
            } );
    }

    private function generateCustomCSS( $file ) {

        $devices = $this->get_enabled_devices();

        if ( ! $devices ) {
            return;
        }

        $css = array_map( function( $device ) {

            return $this->getCssTemplate( $device );
        },
            $devices );

        file_put_contents( $file, implode( PHP_EOL, $css ) );
    }

    private function generateCustomJS( $source_file, $file, $target_file ) {

        $devices = $this->get_enabled_devices();

        if ( ! $devices ) {
            return;
        }

        $devices = array_map( function( $device ) {

            return [
                'name'        => $device[ 'name' ],
                'slug'        => sanitize_title( 'cmb-' . $device[ 'name' ] ),
                'description' => sprintf( __( 'Preview for %s', 'elementor' ), $device[ 'width' ] . 'px' ),
            ];

        },
            $devices );

        file_put_contents( $file, json_encode( array_values( $devices ) ) );

        $js_data = file_get_contents( $source_file );

        $file_url = plugins_url( pathinfo( $file, PATHINFO_BASENAME ), $file );

        $js_data = str_replace( '$$JSON_FILE$$', $file_url, $js_data );
        $js_data = str_replace( '$$$DATE$$$', date( 'Y-m-d H:i:s' ), $js_data );

        file_put_contents( $target_file, $js_data );
    }

    public static function getDeviceClass( $device ) {

        $name = sanitize_title( 'cmb-' . $device[ 'name' ] );

        return $name;
    }

    public static function getDeviceName( $device ) {

        $name = sanitize_title( $device[ 'name' ] );

        return $name;
    }

    private function getCssTemplate( $device ) {

        $name   = $this->getDeviceClass( $device );
        $width  = intval( $device[ 'width' ] );
        $height = intval( $device[ 'height' ] );

        $css = [];

        $css[] = sprintf( '.elementor-device-%s #elementor-preview-responsive-wrapper {', $name );
        $css[] = sprintf( '    width: %dpx;', $width );
        $css[] = sprintf( '    height: %dpx;', $height );
        $css[] = '    padding: 40px 10px 70px;';
        $css[] = '}';

        $css[] = sprintf( '.elementor-device-%1$s .codevision-elementor-cmb.elementor-responsive-switcher.elementor-responsive-switcher-%1$s {', $name );
        $css[] = '    height: 20px;';
        $css[] = '    transform: scale(1); -webkit-transform: scale(1); -ms-transform: scale(1);';
        $css[] = '    opacity: 1';
        $css[] = '}';

        $css[] = sprintf( 'body:not(.elementor-device-%1$s) .elementor-control.elementor-control-responsive-%1$s {', $name );
        $css[] = '    display: none;';
        $css[] = '}';

        return implode( PHP_EOL, $css );
    }

    public function postSaveAction() {

        $target_css_file  = CMB_ROOT . '/admin/css/codevision-elementor-cmb-custom.css';
        $source_js_file   = CMB_ROOT . '/admin/js/' . CMB_SOURCE_JS;
        $target_js_file   = CMB_ROOT . '/admin/js/codevision-elementor-cmb-admin.js';
        $target_json_file = CMB_ROOT . '/admin/js/codevision-elementor-devices.json';

        $this->generateCustomCSS( $target_css_file );
        $this->generateCustomJS( $source_js_file, $target_json_file, $target_js_file );

        set_transient( 'cmb-generated-dynamic-css-and-js', time(), 5 * HOUR_IN_SECONDS );

        return $this;
    }

    private function getStoredData( $key, $post_id ) {

        $meta_data = get_post_meta( $post_id, $key, true );

        if ( ! $meta_data ) {
            return [];
        }

        return $meta_data;
    }

    private function saveDeviceData( $deviceName, $data ) {

        $key       = sprintf( 'cmb-%s-%s', $data[ 'type' ], sha1( $deviceName . $data[ 'post_id' ] . $data[ 'element_id' ] ) );
        $device_id = sprintf( '%s-%s-%s', $deviceName, $data[ 'type' ], $data[ 'element_id' ] );

        update_post_meta( $data[ 'post_id' ], $key, $data );

        $elementData = $this->getStoredData( 'cmb-element-data', $data[ 'post_id' ] );

        $elementData[ $device_id ] = $key;

        update_post_meta( $data[ 'post_id' ], 'cmb-element-data', $elementData );
    }

    private function extendAndSort( $settings ) {

        $devices = $this->get_enabled_devices();

        $extended = array_map( function( $setting ) use ( $devices ) {

            $device = $this->get_device_by_setting( $setting, $devices );

            if ( $device === false ) {
                return $setting;
            }

            $setting[ 'device_width' ]  = $device[ 'width' ];
            $setting[ 'device_height' ] = $device[ 'height' ];

            return $setting;
        },
            $settings );

        uasort( $extended,
            function( $s1, $s2 ) {

                return intval( $s2[ 'device_width' ] ?? 0 ) <=> intval( $s1[ 'device_width' ] ?? 0 );
            } );

        return $extended;
    }

    private function updateCssFile( $postId ) {

        $file     = sprintf( '%s/elementor-%s.css', CMB_PUBLIC_CSS, $postId );
        $settings = $this->getStoredData( 'cmb-element-data', $postId );

        if ( ! $settings && file_exists( $file ) ) {
            @unlink( $file );

            return;
        }

        $settings = array_map( function( $key ) use ( $postId ) {

            return $this->getStoredData( $key, $postId );
        },
            $settings );

        $settings = $this->extendAndSort( $settings );

        $css = array_map( function( $setting ) use ( $postId ) {

            $data = apply_filters( 'codevision/cmb/mixins/render_css', $setting );

            if ( ! is_string( $data ) ) {
                return '';
            }

            return $data;
        },
            $settings );

        if ( ! $css ) {
            return;
        }

        file_put_contents( $file, implode( '', $css ) );
    }

    public static function isElementorPreview() {

        return isset( $_REQUEST[ 'elementor-preview' ] );
    }
}
