<?php


namespace Codevision\Elementor\Mixin;


use Codevision\Elementor\CustomBreakpoints;
use StringTemplate\Engine;

class MarginMixin {

    private $devices;

    public function hooks() {

        add_action( 'wp_ajax_cmb-load-margin', [ $this, 'ajaxLoadMargin' ] );
        add_filter( 'codevision/cmb/mixins/settings', [ $this, 'filterLoadCss' ] );
        add_filter( 'codevision/cmb/mixins', [ $this, 'addMixin' ] );
        add_filter( 'codevision/cmb/mixins/render_css', [ $this, 'renderCss' ] );
    }

    function addMixin( $mixins ) {

        $mixins[] = 'margin';

        return $mixins;
    }

    function renderCss( $setting ) {

        if ( $setting[ 'type' ] !== 'margin' ) {
            return $setting;
        }

        return $this->render_css( $setting );
    }

    function filterLoadCss( $setting ) {

        if ( $setting[ 'type' ] !== 'margin' ) {
            return $setting;
        }

        $setting[ 'rendered_css_code' ] = $this->render_css( $setting );

        return $setting;
    }


    function ajaxLoadMargin() {

        check_ajax_referer( 'cmb-load-mixins', 'security' );

        $postId    = intval( sanitize_text_field( $_POST[ 'post_id' ] ?? '' ) );
        $elementId = sanitize_text_field( $_POST[ 'element_id' ] ?? '' );

        $template = $this->load_template( $postId, $elementId );

        wp_send_json( [
            'template' => $template,
            'script'   => plugin_dir_url( CMB_TEMPLATE_DIR . '/mixin.margin.js' ) . 'mixin.margin.js',
        ] );
    }

    function load_template( $postId, $elementId ) {

        $devices  = $this->get_enabled_devices();
        $defaults = $this->get_configuration( [ 'post_id' => $postId, 'element_id' => $elementId ] );

        $engine = new Engine( '{{', '}}' );

        $html = [];
        foreach ( $devices as $device ) {
            $templateContent = $this->load_template_file();

            if ( $templateContent === false ) {
                continue;
            }

            $data   = $this->load_device_data( $device, $defaults );
            $html[] = $engine->render( $templateContent, $data );
        }

        return implode( '', $html );
    }

    private function unit_from_settings($setting) {
        if ($setting['px_checked']) {
            return 'px';
        }

        if ($setting['em_checked']) {
            return 'em';
        }

        if ($setting['rem_checked']) {
            return 'rem';
        }

        return '%';
    }

    private function render_css( $setting ) {

        $devices = $this->get_enabled_devices();

        $css = [];
        foreach ( $devices as $device ) {

            $device_name = CustomBreakpoints::getDeviceName( $device );

            if ( $setting[ 'device_name' ] !== $device_name ) {
                continue;
            }

            $css[] = sprintf( '@media (max-width: %dpx){', $device[ 'width' ] );
            $css[] = sprintf( '.elementor-%s .elementor-element.elementor-element-%s > .elementor-widget-container{', $setting[ 'post_id' ], $setting[ 'element_id' ] );

            $unit  = $this->unit_from_settings($setting);

            foreach ( [ 'top', 'left', 'bottom', 'right' ] as $key ) {
                $css[] = sprintf( 'margin-%s:%s%s;', $key, $setting[ $key ], $unit );
            }
            $css[] = '}';
            $css[] = '}';
            break;
        }

        return implode( '', $css );
    }

    private function load_device_data( $device, $defaults ) {

        $deviceName = CustomBreakpoints::getDeviceName( $device );
        $key        = sprintf( 'cmb-margin-%s', sha1( $deviceName . $defaults[ 'post_id' ] . $defaults[ 'element_id' ] ) );

        $defaults[ 'device' ]      = $device;
        $defaults[ 'device_name' ] = $deviceName;
        $defaults[ 'class' ]       = CustomBreakpoints::getDeviceClass( $device );
        $defaults[ 'uuid' ]        = uniqid();

        $deviceData = array_merge( $defaults, $this->get_stored_data( $key, $defaults[ 'post_id' ] ) );

        return $deviceData;
    }

    private function get_stored_data( $key, $post_id ) {

        $meta_data = get_post_meta( $post_id, $key, true );

        if ( ! $meta_data ) {
            return [];
        }

        return $meta_data;
    }

    private function get_configuration( $postData ) {

        $base_config = array_merge( [
            'type'       => 'margin',
            'uuid'       => '',
            'device'     => '',
            'class'      => '',
            'post_id'    => 0,
            'element_id' => '',
        ],
            $postData );

        $config = array_merge( $base_config,
            [
                'top'                => 0,
                'right'              => 0,
                'bottom'             => 0,
                'left'               => 0,
                'linked'             => '',
                'px_checked'         => 'checked',
                'em_checked'         => '',
                'rem_checked'         => '',
                'percentage_checked' => '',
            ] );

        return $config;
    }

    private function load_template_file() {

        $file = sprintf( '%s/%s.html', CMB_TEMPLATE_DIR, 'margin' );

        if ( ! file_exists( $file ) ) {
            return false;
        }

        return file_get_contents( $file );
    }

    private function get_enabled_devices() {

        if ( $this->devices ) {
            return $this->devices;
        }

        $devices = get_field( 'cmb_breakpoint_configuration', 'options' );

        if ( ! is_array( $devices ) ) {
            return [];
        }

        $this->devices = array_filter( $devices,
            function( $device ) {

                return ( $device[ 'is_enabled' ] ?? false ) === true;
            } );

        return $this->devices;
    }
}
